package io.tswf.minecraft.blockventure.modloader.init

import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import io.tswf.minecraft.blockventure.modloader.api.setup.IClientSetupAware
import io.tswf.minecraft.blockventure.modloader.api.setup.ICommonSetupAware
import io.tswf.minecraft.blockventure.modloader.exception.InvalidModDescriptionException
import io.tswf.minecraft.blockventure.modloader.meta.ModMetadata
import io.tswf.minecraft.blockventure.modloader.meta.ModStatus
import io.tswf.minecraft.blockventure.modloader.scanner.ModsScanner

@Slf4j(category = "BlockVenture")
@CompileStatic
class BlockVentureModLoader {
    static List<ModMetadata> existingMods = new ArrayList<>()

    static void scanMods() {
        log.info("${BlockVentureModLoader.simpleName}.scanMods")
        existingMods = ModsScanner.scanMods()
    }

    static void instanciateMods() {
        log.info("${BlockVentureModLoader.simpleName}.createMods")
        for (def modMeta : existingMods) {
            def modObjectClass = ClassLoader.contextClassloader().loadClass(modMeta.runtimeMeta.modClassNode.javaName)
            modMeta.runtimeMeta.modClass = modObjectClass

            def modInstance = modObjectClass.noArgsConstructor
                .ensureNotNull { new InvalidModDescriptionException("There is no no-args constructor found in class '${modObjectClass.name}'!", modMeta) }
                .newInstance()

            modMeta.runtimeMeta.modInstance = modInstance
            modMeta.runtimeMeta.status = ModStatus.CREATED
        }
    }

    static void initModsCommonSetup() {
        log.info("${BlockVentureModLoader.simpleName}.initModsCommon")

        for (def modMeta : existingMods) {
            def modInstance = modMeta.runtimeMeta.modInstance
            if (modInstance instanceof ICommonSetupAware) {
                modInstance.setupCommon()
            }
        }
    }

    static void initModsClientSetup() {
        log.info("${BlockVentureModLoader.simpleName}.initModsClient")

        for (def modMeta : existingMods) {
            def modInstance = modMeta.runtimeMeta.modInstance
            if (modInstance instanceof IClientSetupAware) {
                modInstance.setupClient()
            }
        }
    }

    static void completeModsInitialization(){
        log.info("${BlockVentureModLoader.simpleName}.completeModsInitialization")

        for (def modMeta : existingMods) {
            modMeta.runtimeMeta.status = ModStatus.INITIALIZED
        }
    }
}
