package io.tswf.minecraft.blockventure.modloader.reader

import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.modloader.api.annotation.BlockVentureMod
import io.tswf.minecraft.blockventure.modloader.meta.ModMetadata
import io.tswf.minecraft.blockventure.modloader.meta.ModRuntimeMetadata
import org.objectweb.asm.tree.ClassNode

@CompileStatic
class ModMetadataReader {

    public static final String DEFAULT_VERSION = "1.0.0-SNAPSHOT"

    public static final String MOD_ID_PARAMETER_NAME = "id"
    public static final String MOD_NAME_PARAMETER_NAME = "name"
    public static final String DESCRIPTION_PARAMETER_NAME = "description"
    public static final String AUTHORS_PARAMETER_NAME = "authors"
    public static final String VERSION_PARAMETER_NAME = "version"

    static ModMetadata readModMetadata(File modFile, ClassNode modClass) {
        def modAnnotation = modClass.getAnnotationByType(BlockVentureMod)
        def modId = modAnnotation.getValueByName(MOD_ID_PARAMETER_NAME, String)
        def modName = modAnnotation.getValueByName(MOD_NAME_PARAMETER_NAME, String) { modId }
        def modDescription = modAnnotation.getValueByName(DESCRIPTION_PARAMETER_NAME, String) { "" }
        def modAuthors = modAnnotation.getValueByName(AUTHORS_PARAMETER_NAME, String[]) { new String[0] }
        def modVersion = modAnnotation.getValueByName(VERSION_PARAMETER_NAME, String) { DEFAULT_VERSION }

        return new ModMetadata(
                id: modId,
                name: modName,
                description: modDescription,
                authors: modAuthors.toList(),
                version: modVersion,
                runtimeMeta: new ModRuntimeMetadata(
                        modClassNode: modClass,
                        fileFromRead: modFile
                )
        )
    }

}
